<?php

/**
 * @file
 * Contains \Drupal\clamav\Scanner
 */

namespace Drupal\clamav;

use \Drupal\Core\File\FileSystem;
use Drupal\clamav\Config;

class Scanner  {
  const FILE_IS_UNCHECKED = -1;
  const FILE_IS_CLEAN = 0;
  const FILE_IS_INFECTED = 1;


  const FILE_IS_SCANNABLE = TRUE;
  const FILE_IS_NOT_SCANNABLE = FALSE;
  const FILE_SCANNABLE_IGNORE = NULL;

  protected $_file = NULL;
  protected $_config = NULL;

  /**
   * Constructor.
   *
   * @param object $file
   *   A file object that implements \Drupal\file\FileInterface.
   */
  public function __construct(\Drupal\file\FileInterface $file) {
    $this->_file = $file;
    $this->_config = new Config;
  }

  /**
   * Check if the anti-virus checks are enabled.
   *
   * @return boolean
   *   TRUE if files should be scanned.
   */
  public function isEnabled() {
    return $this->_config->enabled();
  }

  /**
   * Report if files that have not been scanned can be uploaded.
   *
   * @return boolean
   *    TRUE if unchecked files are permitted.
   */
  public function allowUncheckedFiles() {
    return $this->_config->outage_action() === Config::OUTAGE_ALLOW_UNCHECKED;
  }

  /**
   * Test if a particular file should be scanned by ClamAV.
   *
   * Specific files can be excluded from anti-virus scanning, such as:
   * - Image files
   * - Large files that might take a long time to scan
   * - Files uploaded by trusted administrators
   * - Viruses, intended to be deliberately uploaded to a virus database
   *
   * Files can be excluded from the scans by implementing
   * hook_clamav_file_is_scannable().
   *
   * @see hook_clamav_file_is_scannable().
   *
   * @return boolean
   *    TRUE if a file should be scanned by the anti-virus service. 
   */
  public function isScannable() {
    $file = $this->_file;

    // Check whether this stream-wrapper scheme is scannable.
    if (!empty($file->destination)) {
      $scheme = \Drupal::service('file_system')->uriScheme($file->destination);
    }
    else {
      $scheme = \Drupal::service('file_system')->uriScheme($file->getFileUri());
    }
    $scannable = clamav_scheme_is_scannable($scheme);

    // Iterate each module implementing hook_clamav_file_is_scannable().
    // Modules that do not wish to affact the result should return
    // FILE_SCANNABLE_IGNORE.
    foreach (\Drupal::moduleHandler()->getImplementations('clamav_file_is_scannable') as $module) {
      $result = \Drupal::moduleHandler()->invoke($module, 'clamav_file_is_scannable', array($this->_file));
      if ($result !== self::FILE_SCANNABLE_IGNORE) {
        $scannable = $result;
      }
    }

    return $scannable;
  }

  /**
   * Scan a file for viruses.
   *
   * @return int
   *   - CLAMAV_SCANRESULT_UNCHECKED
   *     The file was not scanned. The ClamAV service may be unavailable.
   *   - CLAMAV_SCANRESULT_CLEAN
   *     The file was scanned, and no infection was found.
   *   - CLAMAV_SCANRESULT_INFECTED
   *     The file was scanned, and found to be infected with a virus.
   */
  public function scan() {
    // Empty files are never infected.
    if ($this->_file->getSize() === 0) {
      return self::FILE_IS_CLEAN;
    }

    switch ($this->_config->scan_mode()) {
      case Config::MODE_EXECUTABLE:
        $scanner = new Scanner\Executable($this->_config);
        $result = $scanner->scan($this->_file);
        break;

      case Config::MODE_DAEMON:
        $scanner = new Scanner\DaemonTCPIP($this->_config);
        $result = $scanner->scan($this->_file);
        break;

      case Config::MODE_UNIX_SOCKET:
        $scanner = new Scanner\DaemonUnixSocket($this->_config);
        $result = $scanner->scan($this->_file);
        break;
    }

    // Prepare to log results.
    $verbose_mode = $this->_config->verbosity();
    $replacements = array(
      '%filename' => $this->_file->getFileUri(),
      '%virusname' => $scanner->virus_name(),
    );

    switch ($result) {
      // Log every infected file.
      case self::FILE_IS_INFECTED:
        $message = 'Virus %virusname detected in uploaded file %filename.';
        \Drupal::logger('Clam AV')->error($message, $replacements);
        break;

      // Log clean files if verbose mode is enabled.
      case self::FILE_IS_CLEAN:
        if ($verbose_mode) {
          $message = 'Uploaded file %filename checked and found clean.';
          \Drupal::logger('Clam AV')->info($message, $replacements);
        }
        break;

      // Log unchecked files if they are accepted, or verbose mode is enabled.
      case self::FILE_IS_UNCHECKED:
        if ($this->_config->outage_action() === Config::OUTAGE_ALLOW_UNCHECKED) {
          $message = 'Uploaded file %filename could not be checked, and was uploaded without checking.';
          \Drupal::logger('Clam AV')->notice($message, $replacements);
        }
        elseif ($verbose_mode) {
          $message = 'Uploaded file %filename could not be checked, and was deleted.';
          \Drupal::logger('Clam AV')->info($message, $replacements);
        }
        break;
    }

    return $result;
  }

  /**
   * Report if verbose mode is enabled.
   *
   * @return boolean
   *   TRUE if verbose logging is enabled.
   */
  function verbose_mode_is_enabled() {
    return $this->_config->verbosity();
  }
}
